// Small header computed by CMake to set up boost test.
// include AFTER #define BOOST_TEST_MODULE whatever
// but before any other boost test includes.

// Using the Boost UTF included framework

#include <boost/test/included/unit_test.hpp>
